@@ -109,10 +109,9 @@
 /*!
  * To make many functions be inline macros (via \#define) define this.
  * If it is undefined, a function will be used.
-  */
+ */
 /* #define ISC_BUFFER_USEINLINE */
 
-
 ISC_LANG_BEGINDECLS
 
 /*@{*/
@@ -748,6 +747,38 @@ isc_buffer_dup(isc_mem_t *mctx, isc_buffer_t **dstp, const isc_buffer_t *src);
  *					big enough.
  */
 
+isc_result_t
+isc_buffer_printf(isc_buffer_t *b, const char *format, ...)
+	ISC_FORMAT_PRINTF(2, 3);
+/*!<
+ * \brief Append a formatted string to the used region of 'b'.
+ *
+ * Notes:
+ *
+ *\li	The 'format' argument is a printf(3) string, with additional arguments
+ *	as necessary.
+ *
+ *\li	If 'b' has autoreallocation enabled, and the formatted string
+ *	would overrun the buffer, the buffer is reallocated.
+ *
+ * Requires:
+ *
+ *\li	'b' is a valid buffer.
+ *
+ * Ensures:
+ *
+ *\li	The used pointer in 'b' is advanced by the number of bytes appended
+ *	(excluding the terminating NULL byte).
+ *
+ * Returns:
+ *
+ *\li	#ISC_R_SUCCESS	Operation succeeded.
+ *\li	#ISC_R_NOSPACE	'b' does not allow reallocation and appending the
+ *			formatted string to it would cause it to overflow.
+ *\li	#ISC_R_NOMEMORY	Reallocation failed.
+ *\li	#ISC_R_FAILURE	Other error occurred.
+ */
+
 ISC_LANG_ENDDECLS
 
 /*
@@ -940,8 +971,8 @@ ISC_LANG_ENDDECLS
 		ISC_REQUIRE(isc_buffer_availablelength(_b) >= 2U); \
 		_cp = isc_buffer_used(_b); \
 		(_b)->used += 2; \
-		_cp[0] = _val2 >> 8; \
-		_cp[1] = _val2; \
+		_cp[0] = (unsigned char)(_val2 >> 8); \
+		_cp[1] = (unsigned char)_val2; \
 	} while (0)
 
 #define ISC__BUFFER_PUTUINT24(_b, _val) \
@@ -957,9 +988,9 @@ ISC_LANG_ENDDECLS
 		ISC_REQUIRE(isc_buffer_availablelength(_b) >= 3U); \
 		_cp = isc_buffer_used(_b); \
 		(_b)->used += 3; \
-		_cp[0] = _val2 >> 16; \
-		_cp[1] = _val2 >> 8; \
-		_cp[2] = _val2; \
+		_cp[0] = (unsigned char)(_val2 >> 16); \
+		_cp[1] = (unsigned char)(_val2 >> 8); \
+		_cp[2] = (unsigned char)_val2; \
 	} while (0)
 
 #define ISC__BUFFER_PUTUINT32(_b, _val) \
@@ -975,10 +1006,10 @@ ISC_LANG_ENDDECLS
 		ISC_REQUIRE(isc_buffer_availablelength(_b) >= 4U); \
 		_cp = isc_buffer_used(_b); \
 		(_b)->used += 4; \
-		_cp[0] = _val2 >> 24; \
-		_cp[1] = _val2 >> 16; \
-		_cp[2] = _val2 >> 8; \
-		_cp[3] = _val2; \
+		_cp[0] = (unsigned char)(_val2 >> 24); \
+		_cp[1] = (unsigned char)(_val2 >> 16); \
+		_cp[2] = (unsigned char)(_val2 >> 8); \
+		_cp[3] = (unsigned char)_val2; \
 	} while (0)
 
 #if defined(ISC_BUFFER_USEINLINE)
