@@ -1,21 +1,14 @@
 /*
- * Copyright (C) 2005-2007, 2009, 2011, 2012, 2014  Internet Systems Consortium, Inc. ("ISC")
+ * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
  *
- * Permission to use, copy, modify, and/or distribute this software for any
- * purpose with or without fee is hereby granted, provided that the above
- * copyright notice and this permission notice appear in all copies.
+ * This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/.
  *
- * THE SOFTWARE IS PROVIDED "AS IS" AND ISC DISCLAIMS ALL WARRANTIES WITH
- * REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
- * AND FITNESS.  IN NO EVENT SHALL ISC BE LIABLE FOR ANY SPECIAL, DIRECT,
- * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
- * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
- * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
- * PERFORMANCE OF THIS SOFTWARE.
+ * See the COPYRIGHT file distributed with this work for additional
+ * information regarding copyright ownership.
  */
 
-/* $Id$ */
-
 /*	$FreeBSD: src/sys/crypto/sha2/sha2.c,v 1.2.2.2 2002/03/05 08:36:47 ume Exp $	*/
 /*	$KAME: sha2.c,v 1.8 2001/11/08 01:07:52 itojun Exp $	*/
 
@@ -59,6 +52,7 @@
 
 #include <isc/assertions.h>
 #include <isc/platform.h>
+#include <isc/safe.h>
 #include <isc/sha2.h>
 #include <isc/string.h>
 #include <isc/util.h>
@@ -68,19 +62,29 @@
 #include <pk11/pk11.h>
 #endif
 
-#ifdef ISC_PLATFORM_OPENSSLHASH
+#if defined(ISC_PLATFORM_OPENSSLHASH) && !defined(LIBRESSL_VERSION_NUMBER)
+#if OPENSSL_VERSION_NUMBER < 0x10100000L
+#define EVP_MD_CTX_new() &(context->_ctx)
+#define EVP_MD_CTX_free(ptr) EVP_MD_CTX_cleanup(ptr)
+#define EVP_MD_CTX_reset(c) EVP_MD_CTX_cleanup(c)
+#endif
 
 void
 isc_sha224_init(isc_sha224_t *context) {
 	if (context == (isc_sha224_t *)0) {
 		return;
 	}
-	EVP_DigestInit(context, EVP_sha224());
+	context->ctx = EVP_MD_CTX_new();
+	RUNTIME_CHECK(context->ctx != NULL);
+	if (EVP_DigestInit(context->ctx, EVP_sha224()) != 1) {
+		FATAL_ERROR(__FILE__, __LINE__, "Cannot initialize SHA224.");
+	}
 }
 
 void
 isc_sha224_invalidate(isc_sha224_t *context) {
-	EVP_MD_CTX_cleanup(context);
+	EVP_MD_CTX_free(context->ctx);
+	context->ctx = NULL;
 }
 
 void
@@ -91,22 +95,26 @@ isc_sha224_update(isc_sha224_t *context, const isc_uint8_t* data, size_t len) {
 	}
 
 	/* Sanity check: */
-	REQUIRE(context != (isc_sha224_t *)0 && data != (isc_uint8_t*)0);
+	REQUIRE(context != (isc_sha224_t *)0);
+	REQUIRE(context->ctx != (EVP_MD_CTX *)0);
+	REQUIRE(data != (isc_uint8_t*)0);
 
-	EVP_DigestUpdate(context, (const void *) data, len);
+	RUNTIME_CHECK(EVP_DigestUpdate(context->ctx,
+				       (const void *) data, len) == 1);
 }
 
 void
 isc_sha224_final(isc_uint8_t digest[], isc_sha224_t *context) {
 	/* Sanity check: */
 	REQUIRE(context != (isc_sha224_t *)0);
+	REQUIRE(context->ctx != (EVP_MD_CTX *)0);
 
 	/* If no digest buffer is passed, we don't bother doing this: */
-	if (digest != (isc_uint8_t*)0) {
-		EVP_DigestFinal(context, digest, NULL);
-	} else {
-		EVP_MD_CTX_cleanup(context);
-	}
+	if (digest != (isc_uint8_t*)0)
+		RUNTIME_CHECK(EVP_DigestFinal(context->ctx,
+					      digest, NULL) == 1);
+	EVP_MD_CTX_free(context->ctx);
+	context->ctx = NULL;
 }
 
 void
@@ -114,12 +122,17 @@ isc_sha256_init(isc_sha256_t *context) {
 	if (context == (isc_sha256_t *)0) {
 		return;
 	}
-	EVP_DigestInit(context, EVP_sha256());
+	context->ctx = EVP_MD_CTX_new();
+	RUNTIME_CHECK(context->ctx != NULL);
+	if (EVP_DigestInit(context->ctx, EVP_sha256()) != 1) {
+		FATAL_ERROR(__FILE__, __LINE__, "Cannot initialize SHA256.");
+	}
 }
 
 void
 isc_sha256_invalidate(isc_sha256_t *context) {
-	EVP_MD_CTX_cleanup(context);
+	EVP_MD_CTX_free(context->ctx);
+	context->ctx = NULL;
 }
 
 void
@@ -130,22 +143,26 @@ isc_sha256_update(isc_sha256_t *context, const isc_uint8_t *data, size_t len) {
 	}
 
 	/* Sanity check: */
-	REQUIRE(context != (isc_sha256_t *)0 && data != (isc_uint8_t*)0);
+	REQUIRE(context != (isc_sha256_t *)0);
+	REQUIRE(context->ctx != (EVP_MD_CTX *)0);
+	REQUIRE(data != (isc_uint8_t*)0);
 
-	EVP_DigestUpdate(context, (const void *) data, len);
+	RUNTIME_CHECK(EVP_DigestUpdate(context->ctx,
+				       (const void *) data, len) == 1);
 }
 
 void
 isc_sha256_final(isc_uint8_t digest[], isc_sha256_t *context) {
 	/* Sanity check: */
 	REQUIRE(context != (isc_sha256_t *)0);
+	REQUIRE(context->ctx != (EVP_MD_CTX *)0);
 
 	/* If no digest buffer is passed, we don't bother doing this: */
-	if (digest != (isc_uint8_t*)0) {
-		EVP_DigestFinal(context, digest, NULL);
-	} else {
-		EVP_MD_CTX_cleanup(context);
-	}
+	if (digest != (isc_uint8_t*)0)
+		RUNTIME_CHECK(EVP_DigestFinal(context->ctx,
+					      digest, NULL) == 1);
+	EVP_MD_CTX_free(context->ctx);
+	context->ctx = NULL;
 }
 
 void
@@ -153,12 +170,17 @@ isc_sha512_init(isc_sha512_t *context) {
 	if (context == (isc_sha512_t *)0) {
 		return;
 	}
-	EVP_DigestInit(context, EVP_sha512());
+	context->ctx = EVP_MD_CTX_new();
+	RUNTIME_CHECK(context->ctx != NULL);
+	if (EVP_DigestInit(context->ctx, EVP_sha512()) != 1) {
+		FATAL_ERROR(__FILE__, __LINE__, "Cannot initialize SHA512.");
+	}
 }
 
 void
 isc_sha512_invalidate(isc_sha512_t *context) {
-	EVP_MD_CTX_cleanup(context);
+	EVP_MD_CTX_free(context->ctx);
+	context->ctx = NULL;
 }
 
 void isc_sha512_update(isc_sha512_t *context, const isc_uint8_t *data, size_t len) {
@@ -168,21 +190,25 @@ void isc_sha512_update(isc_sha512_t *context, const isc_uint8_t *data, size_t le
 	}
 
 	/* Sanity check: */
-	REQUIRE(context != (isc_sha512_t *)0 && data != (isc_uint8_t*)0);
+	REQUIRE(context != (isc_sha512_t *)0);
+	REQUIRE(context->ctx != (EVP_MD_CTX *)0);
+	REQUIRE(data != (isc_uint8_t*)0);
 
-	EVP_DigestUpdate(context, (const void *) data, len);
+	RUNTIME_CHECK(EVP_DigestUpdate(context->ctx,
+				       (const void *) data, len) == 1);
 }
 
 void isc_sha512_final(isc_uint8_t digest[], isc_sha512_t *context) {
 	/* Sanity check: */
 	REQUIRE(context != (isc_sha512_t *)0);
+	REQUIRE(context->ctx != (EVP_MD_CTX *)0);
 
 	/* If no digest buffer is passed, we don't bother doing this: */
-	if (digest != (isc_uint8_t*)0) {
-		EVP_DigestFinal(context, digest, NULL);
-	} else {
-		EVP_MD_CTX_cleanup(context);
-	}
+	if (digest != (isc_uint8_t*)0)
+		RUNTIME_CHECK(EVP_DigestFinal(context->ctx,
+					      digest, NULL) == 1);
+	EVP_MD_CTX_free(context->ctx);
+	context->ctx = NULL;
 }
 
 void
@@ -190,12 +216,17 @@ isc_sha384_init(isc_sha384_t *context) {
 	if (context == (isc_sha384_t *)0) {
 		return;
 	}
-	EVP_DigestInit(context, EVP_sha384());
+	context->ctx = EVP_MD_CTX_new();
+	RUNTIME_CHECK(context->ctx != NULL);
+	if (EVP_DigestInit(context->ctx, EVP_sha384()) != 1) {
+		FATAL_ERROR(__FILE__, __LINE__, "Cannot initialize SHA384.");
+	}
 }
 
 void
 isc_sha384_invalidate(isc_sha384_t *context) {
-	EVP_MD_CTX_cleanup(context);
+	EVP_MD_CTX_free(context->ctx);
+	context->ctx = NULL;
 }
 
 void
@@ -206,22 +237,26 @@ isc_sha384_update(isc_sha384_t *context, const isc_uint8_t* data, size_t len) {
 	}
 
 	/* Sanity check: */
-	REQUIRE(context != (isc_sha512_t *)0 && data != (isc_uint8_t*)0);
+	REQUIRE(context != (isc_sha512_t *)0);
+	REQUIRE(context->ctx != (EVP_MD_CTX *)0);
+	REQUIRE(data != (isc_uint8_t*)0);
 
-	EVP_DigestUpdate(context, (const void *) data, len);
+	RUNTIME_CHECK(EVP_DigestUpdate(context->ctx,
+				       (const void *) data, len) == 1);
 }
 
 void
 isc_sha384_final(isc_uint8_t digest[], isc_sha384_t *context) {
 	/* Sanity check: */
 	REQUIRE(context != (isc_sha384_t *)0);
+	REQUIRE(context->ctx != (EVP_MD_CTX *)0);
 
 	/* If no digest buffer is passed, we don't bother doing this: */
-	if (digest != (isc_uint8_t*)0) {
-		EVP_DigestFinal(context, digest, NULL);
-	} else {
-		EVP_MD_CTX_cleanup(context);
-	}
+	if (digest != (isc_uint8_t*)0)
+		RUNTIME_CHECK(EVP_DigestFinal(context->ctx,
+					      digest, NULL) == 1);
+	EVP_MD_CTX_free(context->ctx);
+	context->ctx = NULL;
 }
 
 #elif PKCS11CRYPTO
@@ -247,7 +282,7 @@ isc_sha224_invalidate(isc_sha224_t *context) {
 	if (context->handle == NULL)
 		return;
 	(void) pkcs_C_DigestFinal(context->session, garbage, &len);
-	memset(garbage, 0, sizeof(garbage));
+	isc_safe_memwipe(garbage, sizeof(garbage));
 	pk11_return_session(context);
 }
 
@@ -287,7 +322,7 @@ isc_sha224_final(isc_uint8_t digest[], isc_sha224_t *context) {
 		CK_BYTE garbage[ISC_SHA224_DIGESTLENGTH];
 
 		(void) pkcs_C_DigestFinal(context->session, garbage, &len);
-		memset(garbage, 0, sizeof(garbage));
+		isc_safe_memwipe(garbage, sizeof(garbage));
 	}
 	pk11_return_session(context);
 }
@@ -313,7 +348,7 @@ isc_sha256_invalidate(isc_sha256_t *context) {
 	if (context->handle == NULL)
 		return;
 	(void) pkcs_C_DigestFinal(context->session, garbage, &len);
-	memset(garbage, 0, sizeof(garbage));
+	isc_safe_memwipe(garbage, sizeof(garbage));
 	pk11_return_session(context);
 }
 
@@ -353,7 +388,7 @@ isc_sha256_final(isc_uint8_t digest[], isc_sha256_t *context) {
 		CK_BYTE garbage[ISC_SHA256_DIGESTLENGTH];
 
 		(void) pkcs_C_DigestFinal(context->session, garbage, &len);
-		memset(garbage, 0, sizeof(garbage));
+		isc_safe_memwipe(garbage, sizeof(garbage));
 	}
 	pk11_return_session(context);
 }
@@ -379,7 +414,7 @@ isc_sha512_invalidate(isc_sha512_t *context) {
 	if (context->handle == NULL)
 		return;
 	(void) pkcs_C_DigestFinal(context->session, garbage, &len);
-	memset(garbage, 0, sizeof(garbage));
+	isc_safe_memwipe(garbage, sizeof(garbage));
 	pk11_return_session(context);
 }
 
@@ -419,7 +454,7 @@ isc_sha512_final(isc_uint8_t digest[], isc_sha512_t *context) {
 		CK_BYTE garbage[ISC_SHA512_DIGESTLENGTH];
 
 		(void) pkcs_C_DigestFinal(context->session, garbage, &len);
-		memset(garbage, 0, sizeof(garbage));
+		isc_safe_memwipe(garbage, sizeof(garbage));
 	}
 	pk11_return_session(context);
 }
@@ -445,7 +480,7 @@ isc_sha384_invalidate(isc_sha384_t *context) {
 	if (context->handle == NULL)
 		return;
 	(void) pkcs_C_DigestFinal(context->session, garbage, &len);
-	memset(garbage, 0, sizeof(garbage));
+	isc_safe_memwipe(garbage, sizeof(garbage));
 	pk11_return_session(context);
 }
 
@@ -485,7 +520,7 @@ isc_sha384_final(isc_uint8_t digest[], isc_sha384_t *context) {
 		CK_BYTE garbage[ISC_SHA384_DIGESTLENGTH];
 
 		(void) pkcs_C_DigestFinal(context->session, garbage, &len);
-		memset(garbage, 0, sizeof(garbage));
+		isc_safe_memwipe(garbage, sizeof(garbage));
 	}
 	pk11_return_session(context);
 }
@@ -839,7 +874,7 @@ isc_sha224_init(isc_sha224_t *context) {
 
 void
 isc_sha224_invalidate(isc_sha224_t *context) {
-	memset(context, 0, sizeof(isc_sha224_t));
+	isc_safe_memwipe(context, sizeof(*context));
 }
 
 void
@@ -852,7 +887,7 @@ isc_sha224_final(isc_uint8_t digest[], isc_sha224_t *context) {
 	isc_uint8_t sha256_digest[ISC_SHA256_DIGESTLENGTH];
 	isc_sha256_final(sha256_digest, (isc_sha256_t *)context);
 	memmove(digest, sha256_digest, ISC_SHA224_DIGESTLENGTH);
-	memset(sha256_digest, 0, ISC_SHA256_DIGESTLENGTH);
+	isc_safe_memwipe(sha256_digest, sizeof(sha256_digest));
 }
 
 /*** SHA-256: *********************************************************/
@@ -869,7 +904,7 @@ isc_sha256_init(isc_sha256_t *context) {
 
 void
 isc_sha256_invalidate(isc_sha256_t *context) {
-	memset(context, 0, sizeof(isc_sha256_t));
+	isc_safe_memwipe(context, sizeof(*context));
 }
 
 #ifdef ISC_SHA2_UNROLL_TRANSFORM
@@ -1176,7 +1211,7 @@ isc_sha256_final(isc_uint8_t digest[], isc_sha256_t *context) {
 	}
 
 	/* Clean up state data: */
-	memset(context, 0, sizeof(*context));
+	isc_safe_memwipe(context, sizeof(*context));
 	usedspace = 0;
 	POST(usedspace);
 }
@@ -1195,7 +1230,7 @@ isc_sha512_init(isc_sha512_t *context) {
 
 void
 isc_sha512_invalidate(isc_sha512_t *context) {
-	memset(context, 0, sizeof(isc_sha512_t));
+	isc_safe_memwipe(context, sizeof(*context));
 }
 
 #ifdef ISC_SHA2_UNROLL_TRANSFORM
@@ -1500,7 +1535,7 @@ void isc_sha512_final(isc_uint8_t digest[], isc_sha512_t *context) {
 	}
 
 	/* Zero out state data */
-	memset(context, 0, sizeof(*context));
+	isc_safe_memwipe(context, sizeof(*context));
 }
 
 
@@ -1518,7 +1553,7 @@ isc_sha384_init(isc_sha384_t *context) {
 
 void
 isc_sha384_invalidate(isc_sha384_t *context) {
-	memset(context, 0, sizeof(isc_sha384_t));
+	isc_safe_memwipe(context, sizeof(*context));
 }
 
 void
@@ -1553,7 +1588,7 @@ isc_sha384_final(isc_uint8_t digest[], isc_sha384_t *context) {
 	}
 
 	/* Zero out state data */
-	memset(context, 0, sizeof(*context));
+	isc_safe_memwipe(context, sizeof(*context));
 }
 #endif /* !ISC_PLATFORM_OPENSSLHASH */
 
@@ -1581,15 +1616,15 @@ isc_sha224_end(isc_sha224_t *context, char buffer[]) {
 		}
 		*buffer = (char)0;
 	} else {
-#ifdef ISC_PLATFORM_OPENSSLHASH
-		EVP_MD_CTX_cleanup(context);
+#if defined(ISC_PLATFORM_OPENSSLHASH) && !defined(LIBRESSL_VERSION_NUMBER)
+		EVP_MD_CTX_reset(context->ctx);
 #elif PKCS11CRYPTO
 		pk11_return_session(context);
 #else
-		memset(context, 0, sizeof(*context));
+		isc_safe_memwipe(context, sizeof(*context));
 #endif
 	}
-	memset(digest, 0, ISC_SHA224_DIGESTLENGTH);
+	isc_safe_memwipe(digest, sizeof(digest));
 	return buffer;
 }
 
@@ -1622,15 +1657,15 @@ isc_sha256_end(isc_sha256_t *context, char buffer[]) {
 		}
 		*buffer = (char)0;
 	} else {
-#ifdef ISC_PLATFORM_OPENSSLHASH
-		EVP_MD_CTX_cleanup(context);
+#if defined(ISC_PLATFORM_OPENSSLHASH) && !defined(LIBRESSL_VERSION_NUMBER)
+		EVP_MD_CTX_reset(context->ctx);
 #elif PKCS11CRYPTO
 		pk11_return_session(context);
 #else
-		memset(context, 0, sizeof(*context));
+		isc_safe_memwipe(context, sizeof(*context));
 #endif
 	}
-	memset(digest, 0, ISC_SHA256_DIGESTLENGTH);
+	isc_safe_memwipe(digest, sizeof(digest));
 	return buffer;
 }
 
@@ -1663,15 +1698,15 @@ isc_sha512_end(isc_sha512_t *context, char buffer[]) {
 		}
 		*buffer = (char)0;
 	} else {
-#ifdef ISC_PLATFORM_OPENSSLHASH
-		EVP_MD_CTX_cleanup(context);
+#if defined(ISC_PLATFORM_OPENSSLHASH) && !defined(LIBRESSL_VERSION_NUMBER)
+		EVP_MD_CTX_reset(context->ctx);
 #elif PKCS11CRYPTO
 		pk11_return_session(context);
 #else
-		memset(context, 0, sizeof(*context));
+		isc_safe_memwipe(context, sizeof(*context));
 #endif
 	}
-	memset(digest, 0, ISC_SHA512_DIGESTLENGTH);
+	isc_safe_memwipe(digest, sizeof(digest));
 	return buffer;
 }
 
@@ -1704,15 +1739,15 @@ isc_sha384_end(isc_sha384_t *context, char buffer[]) {
 		}
 		*buffer = (char)0;
 	} else {
-#ifdef ISC_PLATFORM_OPENSSLHASH
-		EVP_MD_CTX_cleanup(context);
+#if defined(ISC_PLATFORM_OPENSSLHASH) && !defined(LIBRESSL_VERSION_NUMBER)
+		EVP_MD_CTX_reset(context->ctx);
 #elif PKCS11CRYPTO
 		pk11_return_session(context);
 #else
-		memset(context, 0, sizeof(*context));
+		isc_safe_memwipe(context, sizeof(*context));
 #endif
 	}
-	memset(digest, 0, ISC_SHA384_DIGESTLENGTH);
+	isc_safe_memwipe(digest, sizeof(digest));
 	return buffer;
 }
 
